@@ -13,11 +13,23 @@
 # limitations under the License.
 
 """Tests for luigi-tools parameters."""
+
+import collections
+import dataclasses
+import json
+import typing
+
+# pylint: disable=empty-docstring
+# pylint: disable=missing-class-docstring
+# pylint: disable=missing-function-docstring
+# pylint: disable=redefined-outer-name
+# pylint: disable=unused-argument
 import warnings
 
 import luigi
 import mock
 import pytest
+from luigi.freezing import FrozenOrderedDict
 
 import luigi_tools.parameter
 import luigi_tools.target
@@ -29,8 +41,10 @@ from .tools import create_empty_file
 
 
 def test_ext_parameter(luigi_tools_working_directory):
+    """Test the `luigi_tools.parameter.ExtParameter` class."""
+
     class TaskExtParameter(luigi.Task):
-        """"""
+        """A simple test task."""
 
         a = luigi_tools.parameter.ExtParameter(default=".ext_from_default")
         test_type = luigi.Parameter(default="default")
@@ -57,8 +71,10 @@ def test_ext_parameter(luigi_tools_working_directory):
 
 
 def test_ratio_parameter(tmpdir):
+    """Test the `luigi_tools.parameter.RatioParameter` class."""
+
     class TaskRatioParameter(luigi.Task):
-        """"""
+        """A simple test task."""
 
         a = luigi_tools.parameter.RatioParameter(default=0.5)
         b = luigi.FloatParameter(default=0.5)
@@ -83,24 +99,34 @@ def test_ratio_parameter(tmpdir):
 
 
 class TestOptionalParameter:
+    """Test the `luigi_tools.parameter.Optional*` classes.
+
+    .. note::
+
+        These classes were ported to the luigi package but the tests were kept here for example
+        purpose.
+    """
+
     def test_optional_parameter(self, luigi_tools_working_directory):
+        """Simple test for optional parameters."""
+        # pylint: disable=too-many-statements
         class TaskFactory:
+            """A class factory."""
+
             def __call__(self):
                 class TaskOptionalParameter(luigi.Task):
-                    """"""
+                    """A simple test task."""
 
-                    a = luigi_tools.parameter.OptionalIntParameter(default=1)
-                    b = luigi_tools.parameter.OptionalFloatParameter(default=1.5)
-                    c = luigi_tools.parameter.OptionalNumericalParameter(
+                    a = luigi.parameter.OptionalIntParameter(default=1)
+                    b = luigi.parameter.OptionalFloatParameter(default=1.5)
+                    c = luigi.parameter.OptionalNumericalParameter(
                         default=0.75, min_value=0, max_value=1, var_type=float
                     )
                     d = luigi_tools.parameter.OptionalRatioParameter(default=0.5)
-                    e = luigi_tools.parameter.OptionalIntParameter(default=None)
-                    f = luigi_tools.parameter.OptionalListParameter(default=None)
-                    g = luigi_tools.parameter.OptionalChoiceParameter(
-                        default=None, choices=["a", "b"]
-                    )
-                    h = luigi_tools.parameter.OptionalStrParameter(default="h")
+                    e = luigi.parameter.OptionalIntParameter(default=None)
+                    f = luigi.parameter.OptionalListParameter(default=None)
+                    g = luigi.parameter.OptionalChoiceParameter(default=None, choices=["a", "b"])
+                    h = luigi.parameter.OptionalStrParameter(default="h")
                     i = luigi_tools.parameter.OptionalBoolParameter(default=None)
                     expected_a = luigi.IntParameter(default=1)
                     expected_b = luigi.FloatParameter(default=1.5)
@@ -149,6 +175,7 @@ class TestOptionalParameter:
                         create_empty_file(self.output().path)
 
                     def output(self):
+                        # pylint: disable=consider-using-f-string
                         return luigi.LocalTarget(
                             luigi_tools_working_directory
                             / (
@@ -318,9 +345,9 @@ class TestOptionalParameter:
                 assert luigi.build([task()], local_scheduler=True)
 
         class TaskOptionalParameterWarning(luigi.Task):
-            """"""
+            """A simple test task."""
 
-            a = luigi_tools.parameter.OptionalIntParameter(default=1)
+            a = luigi.parameter.OptionalIntParameter(default=1)
             expected_a = luigi.IntParameter(default=1)
 
             def run(self):
@@ -340,7 +367,7 @@ class TestOptionalParameter:
             )
             warnings.simplefilter(
                 action="always",
-                category=luigi_tools.parameter.OptionalParameterTypeWarning,
+                category=luigi.parameter.OptionalParameterTypeWarning,
             )
             assert luigi.build(
                 [TaskOptionalParameterWarning(a="zz", expected_a="zz")],
@@ -348,27 +375,31 @@ class TestOptionalParameter:
             )
 
             assert len(w) == 1
-            assert issubclass(w[0].category, luigi_tools.parameter.OptionalParameterTypeWarning)
+            assert issubclass(w[0].category, luigi.parameter.OptionalParameterTypeWarning)
             assert str(w[0].message) == (
-                "OptionalIntParameter 'a' with value 'zz' is not of type 'int' or None."
+                """OptionalIntParameter "a" with value "zz" is not of type "int" or None."""
             )
 
-    def test_warning(current_test):
+    def test_warning(self):
+        """Test that warnings are properly emitted."""
+
         class TestOptionalFloatParameterSingleType(
-            luigi_tools.parameter.OptionalParameter, luigi.FloatParameter
+            luigi.parameter.OptionalParameterMixin, luigi.FloatParameter
         ):
             """Class to parse optional float parameters."""
 
             expected_type = float
 
         class TestOptionalFloatParameterMultiTypes(
-            luigi_tools.parameter.OptionalParameter, luigi.FloatParameter
+            luigi.parameter.OptionalParameterMixin, luigi.FloatParameter
         ):
             """Class to parse optional float parameters."""
 
             expected_type = (int, float)
 
         class TestConfig(luigi.Config):
+            """A simple config task."""
+
             param_single = TestOptionalFloatParameterSingleType()
             param_multi = TestOptionalFloatParameterMultiTypes()
 
@@ -384,26 +415,30 @@ class TestOptionalParameter:
             )
             warnings.simplefilter(
                 action="always",
-                category=luigi_tools.parameter.OptionalParameterTypeWarning,
+                category=luigi.parameter.OptionalParameterTypeWarning,
             )
             assert luigi.build(
                 [TestConfig(param_single="0", param_multi="1")], local_scheduler=True
             )
 
         assert len(record) == 2
-        assert issubclass(record[0].category, luigi_tools.parameter.OptionalParameterTypeWarning)
-        assert issubclass(record[1].category, luigi_tools.parameter.OptionalParameterTypeWarning)
+        assert issubclass(record[0].category, luigi.parameter.OptionalParameterTypeWarning)
+        assert issubclass(record[1].category, luigi.parameter.OptionalParameterTypeWarning)
         assert str(record[0].message) == (
-            """TestOptionalFloatParameterSingleType 'param_single' with value '0' is not of type """
-            """'float' or None."""
+            """TestOptionalFloatParameterSingleType "param_single" with value "0" is not of type """
+            """"float" or None."""
         )
         assert str(record[1].message) == (
-            """TestOptionalFloatParameterMultiTypes 'param_multi' with value '1' is not of any """
-            """type in ['int', 'float'] or None."""
+            """TestOptionalFloatParameterMultiTypes "param_multi" with value "1" is not of any """
+            """type in ["int", "float"] or None."""
         )
 
-    def actual_test(current_test, cls, default, expected_value, expected_type, bad_data, **kwargs):
+    def actual_test(self, cls, default, expected_value, expected_type, bad_data, **kwargs):
+        """A test function used for several parameter types."""
+
         class TestConfig(luigi.Config):
+            """A simple config task."""
+
             param = cls(default=default, **kwargs)
             empty_param = cls(default=default, **kwargs)
 
@@ -415,42 +450,44 @@ class TestOptionalParameter:
         assert cls(**kwargs).parse("") is None
 
         # Test that warning is raised only with bad type
-        with mock.patch("luigi_tools.parameter.warnings") as warnings:
+        with mock.patch("luigi.parameter.warnings") as warnings:
             TestConfig()
             warnings.warn.assert_not_called()
 
-        if cls != luigi_tools.parameter.OptionalChoiceParameter:
-            with mock.patch("luigi_tools.parameter.warnings") as warnings:
+        if cls != luigi.parameter.OptionalChoiceParameter:
+            with mock.patch("luigi.parameter.warnings") as warnings:
                 TestConfig(param=None)
                 warnings.warn.assert_not_called()
 
-            with mock.patch("luigi_tools.parameter.warnings") as warnings:
+            with mock.patch("luigi.parameter.warnings") as warnings:
                 TestConfig(param=bad_data)
                 if cls == luigi_tools.parameter.OptionalBoolParameter:
                     warnings.warn.assert_not_called()
                 else:
                     assert warnings.warn.call_count == 1
+                    # pylint: disable=consider-using-f-string
                     warnings.warn.assert_called_with(
-                        "{} 'param' with value '{}' is not of type '{}' or None.".format(
+                        """{} "param" with value "{}" is not of type "{}" or None.""".format(
                             cls.__name__, bad_data, expected_type
                         ),
-                        luigi_tools.parameter.OptionalParameterTypeWarning,
+                        luigi.parameter.OptionalParameterTypeWarning,
                     )
 
         # Test with value from config
         assert luigi.build([TestConfig()], local_scheduler=True)
 
     def test_optional_str_parameter(self, tmp_working_dir):
+        """Test the `luigi.parameter.OptionalStrParameter` class."""
         with set_luigi_config({"TestConfig": {"param": "expected value", "empty_param": ""}}):
             self.actual_test(
-                luigi_tools.parameter.OptionalStrParameter,
+                luigi.parameter.OptionalStrParameter,
                 None,
                 "expected value",
                 "str",
                 0,
             )
             self.actual_test(
-                luigi_tools.parameter.OptionalStrParameter,
+                luigi.parameter.OptionalStrParameter,
                 "default value",
                 "expected value",
                 "str",
@@ -458,13 +495,13 @@ class TestOptionalParameter:
             )
 
     def test_optional_int_parameter(self, tmp_working_dir):
+        """Test the `luigi.parameter.OptionalIntParameter` class."""
         with set_luigi_config({"TestConfig": {"param": "10", "empty_param": ""}}):
-            self.actual_test(
-                luigi_tools.parameter.OptionalIntParameter, None, 10, "int", "bad data"
-            )
-            self.actual_test(luigi_tools.parameter.OptionalIntParameter, 1, 10, "int", "bad data")
+            self.actual_test(luigi.parameter.OptionalIntParameter, None, 10, "int", "bad data")
+            self.actual_test(luigi.parameter.OptionalIntParameter, 1, 10, "int", "bad data")
 
     def test_optional_bool_parameter(self, tmp_working_dir):
+        """Test the `luigi.parameter.OptionalBoolParameter` class."""
         with set_luigi_config({"TestConfig": {"param": "true", "empty_param": ""}}):
             self.actual_test(
                 luigi_tools.parameter.OptionalBoolParameter,
@@ -482,16 +519,17 @@ class TestOptionalParameter:
             )
 
     def test_optional_float_parameter(self, tmp_working_dir):
+        """Test the `luigi.parameter.OptionalFloatParameter` class."""
         with set_luigi_config({"TestConfig": {"param": "10.5", "empty_param": ""}}):
             self.actual_test(
-                luigi_tools.parameter.OptionalFloatParameter,
+                luigi.parameter.OptionalFloatParameter,
                 None,
                 10.5,
                 "float",
                 "bad data",
             )
             self.actual_test(
-                luigi_tools.parameter.OptionalFloatParameter,
+                luigi.parameter.OptionalFloatParameter,
                 1.5,
                 10.5,
                 "float",
@@ -499,16 +537,17 @@ class TestOptionalParameter:
             )
 
     def test_optional_dict_parameter(self, tmp_working_dir):
+        """Test the `luigi.parameter.OptionalDictParameter` class."""
         with set_luigi_config({"TestConfig": {"param": '{"a": 10}', "empty_param": ""}}):
             self.actual_test(
-                luigi_tools.parameter.OptionalDictParameter,
+                luigi.parameter.OptionalDictParameter,
                 None,
                 {"a": 10},
                 "FrozenOrderedDict",
                 "bad data",
             )
             self.actual_test(
-                luigi_tools.parameter.OptionalDictParameter,
+                luigi.parameter.OptionalDictParameter,
                 {"a": 1},
                 {"a": 10},
                 "FrozenOrderedDict",
@@ -516,16 +555,17 @@ class TestOptionalParameter:
             )
 
     def test_optional_list_parameter(self, tmp_working_dir):
+        """Test the `luigi.parameter.OptionalListParameter` class."""
         with set_luigi_config({"TestConfig": {"param": "[10.5]", "empty_param": ""}}):
             self.actual_test(
-                luigi_tools.parameter.OptionalListParameter,
+                luigi.parameter.OptionalListParameter,
                 None,
                 (10.5,),
                 "tuple",
                 "bad data",
             )
             self.actual_test(
-                luigi_tools.parameter.OptionalListParameter,
+                luigi.parameter.OptionalListParameter,
                 (1.5,),
                 (10.5,),
                 "tuple",
@@ -533,16 +573,17 @@ class TestOptionalParameter:
             )
 
     def test_optional_tuple_parameter(self, tmp_working_dir):
+        """Test the `luigi.parameter.OptionalTupleParameter` class."""
         with set_luigi_config({"TestConfig": {"param": "[10.5]", "empty_param": ""}}):
             self.actual_test(
-                luigi_tools.parameter.OptionalTupleParameter,
+                luigi.parameter.OptionalTupleParameter,
                 None,
                 (10.5,),
                 "tuple",
                 "bad data",
             )
             self.actual_test(
-                luigi_tools.parameter.OptionalTupleParameter,
+                luigi.parameter.OptionalTupleParameter,
                 (1.5,),
                 (10.5,),
                 "tuple",
@@ -550,9 +591,10 @@ class TestOptionalParameter:
             )
 
     def test_optional_numerical_parameter(self, tmp_working_dir):
+        """Test the `luigi.parameter.OptionalNumericalParameter` class."""
         with set_luigi_config({"TestConfig": {"param": "10.5", "empty_param": ""}}):
             self.actual_test(
-                luigi_tools.parameter.OptionalNumericalParameter,
+                luigi.parameter.OptionalNumericalParameter,
                 None,
                 10.5,
                 "float",
@@ -562,7 +604,7 @@ class TestOptionalParameter:
                 max_value=100,
             )
             self.actual_test(
-                luigi_tools.parameter.OptionalNumericalParameter,
+                luigi.parameter.OptionalNumericalParameter,
                 1.5,
                 10.5,
                 "float",
@@ -573,10 +615,11 @@ class TestOptionalParameter:
             )
 
     def test_optional_choice_parameter(self, tmp_working_dir):
+        """Test the `luigi.parameter.OptionalChoiceParameter` class."""
         with set_luigi_config({"TestConfig": {"param": "expected value", "empty_param": ""}}):
             choices = ["default value", "expected value", "null"]
             self.actual_test(
-                luigi_tools.parameter.OptionalChoiceParameter,
+                luigi.parameter.OptionalChoiceParameter,
                 None,
                 "expected value",
                 "str",
@@ -584,7 +627,7 @@ class TestOptionalParameter:
                 choices=choices,
             )
             self.actual_test(
-                luigi_tools.parameter.OptionalChoiceParameter,
+                luigi.parameter.OptionalChoiceParameter,
                 "default value",
                 "expected value",
                 "str",
@@ -594,9 +637,13 @@ class TestOptionalParameter:
 
 
 class TestBoolParameter:
+    """Test the `luigi_tools.parameter.BoolParameter` class."""
+
     def test_auto_set_parsing(self):
+        """Test that parsing is properly fixed according to the default value."""
+
         class TaskBoolParameter(luigi.Task):
-            """"""
+            """A simple test task."""
 
             a = luigi_tools.parameter.BoolParameter(default=True)
             b = luigi_tools.parameter.BoolParameter(default=False)
@@ -620,41 +667,43 @@ class TestBoolParameter:
             assert luigi.build([TaskBoolParameter()], local_scheduler=True)
 
     def test_true_implicit_failing(self):
+        """Test that setting `default=True` and `parsing="implicit"` raises an exception."""
+        # pylint: disable=unused-variable
         with pytest.raises(ValueError):
 
             class TaskBoolParameterFail(luigi.Task):
-                """"""
+                """A simple test task."""
 
                 a = luigi_tools.parameter.BoolParameter(default=True, parsing="implicit")
 
 
 @pytest.mark.parametrize("default", [None, "not_existing_dir"])
 @pytest.mark.parametrize("absolute", [True, False])
-@pytest.mark.parametrize("create", [True, False])
 @pytest.mark.parametrize("exists", [True, False])
-def test_path_parameter(tmpdir, default, absolute, create, exists):
-    class TaskPathParameter(luigi.Task):
+def test_path_parameter(tmpdir, default, absolute, exists):
+    """Test the `luigi.parameter.PathParameter` class."""
 
-        a = luigi_tools.parameter.PathParameter(
+    class TaskPathParameter(luigi.Task):
+        """A simple test task."""
+
+        a = luigi.parameter.PathParameter(
             default=str(tmpdir / default) if default is not None else str(tmpdir),
             absolute=absolute,
-            create=create,
             exists=exists,
         )
-        b = luigi_tools.parameter.OptionalPathParameter(
+        b = luigi.parameter.OptionalPathParameter(
             default=str(tmpdir / default) if default is not None else str(tmpdir),
             absolute=absolute,
-            create=create,
             exists=exists,
         )
-        c = luigi_tools.parameter.OptionalPathParameter(default=None)
-        d = luigi_tools.parameter.OptionalPathParameter(default="not empty default")
+        c = luigi.parameter.OptionalPathParameter(default=None)
+        d = luigi.parameter.OptionalPathParameter(default="not empty default")
 
         def run(self):
             # Use the parameter as a Path object
             new_file = self.a / "test.file"
             new_optional_file = self.b / "test_optional.file"
-            if default is not None and not create:
+            if default is not None:
                 new_file.parent.mkdir(parents=True)
             new_file.touch()
             new_optional_file.touch()
@@ -668,7 +717,7 @@ def test_path_parameter(tmpdir, default, absolute, create, exists):
 
     # Test with default values
     with set_luigi_config({"TaskPathParameter": {"d": ""}}):
-        if default is not None and not create and exists:
+        if default is not None and exists:
             with pytest.raises(ValueError, match="The path .* does not exist"):
                 luigi.build([TaskPathParameter()], local_scheduler=True)
         else:
@@ -688,8 +737,417 @@ def test_path_parameter(tmpdir, default, absolute, create, exists):
             }
         }
     ):
-        if default is not None and not create and exists:
+        if default is not None and exists:
             with pytest.raises(ValueError, match="The path .* does not exist"):
                 luigi.build([TaskPathParameter()], local_scheduler=True)
         else:
             assert luigi.build([TaskPathParameter()], local_scheduler=True)
+
+
+def _check_parameter(*, obj, parameter, expected_dict, override_comparison_fields=None):
+    """Check DataclassParameter for simple objects."""
+    string = parameter.serialize(obj)
+    assert string == json.dumps(expected_dict)
+
+    dictionary = parameter.parse(string)
+    assert dictionary == expected_dict
+
+    new_obj1 = parameter.normalize(dictionary)
+    new_obj2 = parameter.normalize(obj)
+
+    for new_obj in (new_obj1, new_obj2):
+        # manually compare the attributes for expected differences with origin obj
+        if override_comparison_fields:
+            for attribute_name, attribute_value in override_comparison_fields.items():
+
+                actual_value = getattr(new_obj, attribute_name)
+                assert actual_value == attribute_value
+        else:
+            # compare recursively
+            _compare(new_obj, obj)
+
+
+def _compare(actual, expected):
+    """Recursively compare the two objects."""
+    if dataclasses.is_dataclass(expected):
+        assert type(actual) is type(expected), (
+            f"Actual Class {type(actual)}. " f"Expected Class {type(expected)}."
+        )
+        for f in dataclasses.fields(expected):
+            actual_value = getattr(actual, f.name)
+            expected_value = getattr(expected, f.name)
+            _compare(actual_value, expected_value)
+    elif isinstance(expected, collections.abc.Mapping):
+        assert isinstance(actual, FrozenOrderedDict)
+        for key in expected:
+            _compare(actual[key], expected[key])
+    elif isinstance(expected, collections.abc.Sequence) and not isinstance(expected, str):
+        assert isinstance(actual, tuple)
+        for i, value in enumerate(expected):
+            _compare(actual[i], value)
+    else:
+        assert actual == expected
+
+
+def test_DataclassParameter__raises_no_dataclass():
+    """Test guard against passing non dataclasses."""
+
+    class A:
+        a: int
+
+    with pytest.raises(TypeError, match=r"Class type \'A\' is not a dataclass."):
+        luigi_tools.parameter.DataclassParameter(cls_type=A)
+
+
+def test_DataclassParameter__primitives():
+    """Test the DataclassParameter with primitive types."""
+
+    @dataclasses.dataclass
+    class MyClass:
+        a: str
+        b: int
+        c: float
+        d: bool
+
+    parameter = luigi_tools.parameter.DataclassParameter(cls_type=MyClass)
+
+    data = {"a": "1", "b": 2, "c": 3.0, "d": False}
+
+    instance = MyClass(**data)
+
+    _check_parameter(obj=instance, parameter=parameter, expected_dict=data)
+
+
+def test_DataclassParameter__primitives__ordering():
+    """Test the DataclassParameter with differing primitive ordering."""
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class A:
+        a: str
+        b: int
+        c: float
+        d: bool
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class B:
+        b: int
+        a: str
+        d: bool
+        c: float
+
+    a = A(a="1", b=2, c=3.0, d=False)
+    b = B(a="1", b=2, c=3.0, d=False)
+
+    pA = luigi_tools.parameter.DataclassParameter(cls_type=A)
+    pB = luigi_tools.parameter.DataclassParameter(cls_type=B)
+
+    # ordering affects serialization
+    sA = pA.serialize(a)
+    sB = pB.serialize(b)
+    assert sA != sB
+
+    # but not dictionary comparison
+    dA = pA.parse(sA)
+    dB = pB.parse(sB)
+    assert dA == dB
+
+    # ordering also affects deserialization and hashes
+    nA = pA.normalize(a)
+    nB = pB.normalize(b)
+    assert nA != nB
+    assert hash(nA) != hash(nB)
+
+    # ordering also affects deserialization and hashes
+    nA = pA.normalize(dA)
+    nB = pB.normalize(dB)
+    assert nA != nB
+    assert hash(nA) != hash(nB)
+
+
+def test_DataclassParameter__sequences():
+    """Test the DataclassParameter with sequence types."""
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class A:
+        a: list
+        b: tuple
+        c: typing.List[str]
+        d: typing.List[int]
+        e: typing.Tuple[float]
+
+    a = A(a=[1, "1"], b=(2.0, "2"), c=["a", "b"], d=[3, 4], e=(5.0, 6.0))
+
+    p = luigi_tools.parameter.DataclassParameter(cls_type=A)
+
+    # Note how the e tuple has been converted to a list with json serialization
+    expected_dict = {"a": [1, "1"], "b": [2.0, "2"], "c": ["a", "b"], "d": [3, 4], "e": [5.0, 6.0]}
+
+    _check_parameter(obj=a, parameter=p, expected_dict=expected_dict)
+
+
+def test_DataclassParameter__mappings():
+    """Test the DataclassParameter with mapping types."""
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class A:
+        a: dict
+        b: collections.OrderedDict
+        c: typing.Dict[str, int]
+        d: typing.Dict[int, float]
+
+    a = A(
+        a={"a": 1, "b": "2"},
+        b=collections.OrderedDict([("c", 3.0), ("d", 4)]),
+        c={"e": 5, "f": 6},
+        d={7: 8.0, 9: 10.0},
+    )
+
+    p = luigi_tools.parameter.DataclassParameter(cls_type=A)
+
+    # Note how json serialization converts d keys into strings
+    expected_dict = {
+        "a": {"a": 1, "b": "2"},
+        "b": {"c": 3.0, "d": 4},
+        "c": {"e": 5, "f": 6},
+        "d": {"7": 8.0, "9": 10.0},
+    }
+    _check_parameter(obj=a, parameter=p, expected_dict=expected_dict)
+
+
+def test_DataclassParameter__nesting():
+    """Test the DataclassParameter with nested dataclass objects."""
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class A:
+        a: str
+        b: float
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class B:
+        c: A
+        d: float
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class C:
+        e: A
+        f: B
+
+    c = C(e=A(a="1", b=2.0), f=B(c=A(a="2", b=3.0), d=4.0))
+
+    p = luigi_tools.parameter.DataclassParameter(cls_type=C)
+
+    expected_dict = {"e": {"a": "1", "b": 2.0}, "f": {"c": {"a": "2", "b": 3.0}, "d": 4.0}}
+
+    _check_parameter(obj=c, parameter=p, expected_dict=expected_dict)
+
+
+def test_DataclassParameter__nesting__2():
+    """Test the DataclassParameter with nested dataclass objects."""
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class A:
+        a: str
+        b: float
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class B:
+        c: A
+        d: float
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class C:
+        e: typing.List[A]
+        f: typing.Dict[str, B]
+
+    p = luigi_tools.parameter.DataclassParameter(cls_type=C)
+
+    obj = C(
+        e=[A(a="1", b=2.0), A(a="2", b=3.0)],
+        f={
+            "4": B(c=A(a="5", b=6.0), d=7.0),
+            "5": B(c=A(a="8", b=9.0), d=10.0),
+        },
+    )
+    _check_parameter(
+        obj=obj,
+        parameter=p,
+        expected_dict={
+            "e": [
+                {"a": "1", "b": 2.0},
+                {"a": "2", "b": 3.0},
+            ],
+            "f": {
+                "4": {"c": {"a": "5", "b": 6.0}, "d": 7.0},
+                "5": {"c": {"a": "8", "b": 9.0}, "d": 10.0},
+            },
+        },
+    )
+
+
+def test_DataclassParameter__Optional__Union():
+    """Test the DataclassParameter with Optional attributes."""
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class A:
+        a: int
+        b: typing.Union[int, str]
+        c: typing.Optional[float] = None
+        d: typing.Optional[typing.List[int]] = None
+        e: typing.Optional[typing.Dict[str, float]] = None
+
+    a1 = A(a=1, b=2, c=3.0, d=[4, 5], e={"6": 7.0, "8": 9.0})
+    a2 = A(a=1, b="2")
+
+    p = luigi_tools.parameter.DataclassParameter(cls_type=A)
+
+    _check_parameter(
+        obj=a1,
+        parameter=p,
+        expected_dict={"a": 1, "b": 2, "c": 3.0, "d": [4, 5], "e": {"6": 7.0, "8": 9.0}},
+    )
+    _check_parameter(
+        obj=a2,
+        parameter=p,
+        expected_dict={"a": 1, "b": "2", "c": None, "d": None, "e": None},
+    )
+
+
+def test_DataclassParameter__Any():
+    """Test the DataclassParameter with typing.Any attributes."""
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class D:
+        a: int
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class A:
+        a: typing.Any
+        b: typing.Any
+        c: typing.List[typing.Any]
+        d: typing.Dict[str, typing.Any]
+        e: typing.Optional[typing.Dict[int, typing.Any]]
+
+    a = A(
+        a="1",
+        b=["1", 2, 3.0, [D(a=4)]],
+        c=[5, "6", 7.0, [D(a=8)]],
+        d={"9": 10.0, "11": "12"},
+        e={13: 14, 15: "16", 17: {"a": D(a=18)}},
+    )
+
+    p = luigi_tools.parameter.DataclassParameter(cls_type=A)
+
+    expected_dict = {
+        "a": "1",
+        "b": ["1", 2, 3.0, [{"a": 4}]],
+        "c": [5, "6", 7.0, [{"a": 8}]],
+        "d": {"9": 10.0, "11": "12"},
+        "e": {"13": 14, "15": "16", "17": {"a": {"a": 18}}},
+    }
+
+    string = p.serialize(a)
+    assert string == json.dumps(expected_dict)
+
+    dictionary = p.parse(string)
+    assert dictionary == expected_dict
+
+    obj = p.normalize(dictionary)
+    assert obj.a == "1"
+
+    # Note: It is not possible to reconstruct the D dataclass if the type is Any
+    assert obj.b == ("1", 2, 3.0, (FrozenOrderedDict([("a", 4)]),))
+    assert obj.c == (5, "6", 7.0, (FrozenOrderedDict([("a", 8)]),))
+    assert obj.d == FrozenOrderedDict(a.d)
+    assert obj.e == FrozenOrderedDict(
+        [
+            (13, 14),
+            (15, "16"),
+            (17, FrozenOrderedDict([("a", FrozenOrderedDict([("a", 18)]))])),
+        ]
+    )
+
+
+def test_DataclassParameter__build():
+    """Test DataclassParameter using a config and luigi build."""
+
+    @dataclasses.dataclass(frozen=True, eq=True)
+    class ParamA:
+        a: dict
+        b: collections.OrderedDict
+        c: typing.Dict[str, int]
+        d: typing.Dict[int, float]
+
+    class TaskDataClassParameter(luigi.Task):
+        """A simple test task."""
+
+        a = luigi_tools.parameter.DataclassParameter(cls_type=ParamA)
+
+        def run(self):
+            # pylint: disable=no-member
+            assert self.a.a == {"a": 1, "b": "2"}
+            assert self.a.b == {"c": 3.0, "d": 4}
+            assert self.a.c == {"e": 5, "f": 6}
+            assert self.a.d == {7: 8.0, 9: 10.0}
+
+        def output(self):
+            return luigi.LocalTarget("not_existing_file")
+
+    # Test with values from config
+    with set_luigi_config(
+        {
+            "TaskDataClassParameter": {
+                "a": """{
+                    "a": {"a": 1, "b": "2"},
+                    "b": {"c": 3.0, "d": 4},
+                    "c": {"e": 5, "f": 6},
+                    "d": {"7": 8.0, "9": 10.0}
+                }"""
+            }
+        }
+    ):
+        assert luigi.build([TaskDataClassParameter()], local_scheduler=True)
+
+    # Parsing fails with integer keys
+    with set_luigi_config(
+        {
+            "TaskDataClassParameter": {
+                "a": """{
+                    "a": {"a": 1, "b": "2"},
+                    "b": {"c": 3.0, "d": 4},
+                    "c": {"e": 5, "f": 6},
+                    "d": {7: 8.0, 9: 10.0}
+                }"""
+            }
+        }
+    ):
+        with pytest.raises(ValueError):
+            luigi.build([TaskDataClassParameter()], local_scheduler=True)
+
+    # But the constructor works with either integers and strings as keys
+    assert luigi.build(
+        [
+            TaskDataClassParameter(
+                {
+                    "a": {"a": 1, "b": "2"},
+                    "b": {"c": 3.0, "d": 4},
+                    "c": {"e": 5, "f": 6},
+                    "d": {7: 8.0, 9: 10.0},
+                }
+            )
+        ],
+        local_scheduler=True,
+    )
+
+    assert luigi.build(
+        [
+            TaskDataClassParameter(
+                {
+                    "a": {"a": 1, "b": "2"},
+                    "b": {"c": 3.0, "d": 4},
+                    "c": {"e": 5, "f": 6},
+                    "d": {"7": 8.0, "9": 10.0},
+                }
+            )
+        ],
+        local_scheduler=True,
+    )
