@@ -13,12 +13,17 @@
 # limitations under the License.
 
 """Tests for luigi-tools utils."""
+
+# pylint: disable=empty-docstring
+# pylint: disable=protected-access
+# pylint: disable=unused-argument
 import copy
 from configparser import ConfigParser
 from pathlib import Path
 
 import luigi
 import pytest
+from graphviz import Digraph
 from luigi.parameter import _no_value as PARAM_NO_VALUE
 
 import luigi_tools
@@ -33,8 +38,10 @@ DATA = Path(__file__).parent / "data"
 
 
 def test_target_remove(tmpdir):
+    """Test the `luigi_tools.util.target_remove` function."""
+
     class TaskA(luigi_tools.task.WorkflowTask):
-        """"""
+        """A simple test task."""
 
         def run(self):
             for i in luigi.task.flatten(self.output()):
@@ -59,13 +66,19 @@ def test_target_remove(tmpdir):
 
 
 def test_apply_over_inputs():
+    """Test the `luigi_tools.util.apply_over_inputs` function."""
+
     class TaskA(luigi.Task):
+        """A simple test task."""
+
         a = luigi.Parameter(default="a")
 
         def output(self):
             return self.a
 
     class TaskB(luigi.Task):
+        """A simple test task."""
+
         def requires(self):
             return {
                 "a": TaskA(),
@@ -82,7 +95,11 @@ def test_apply_over_inputs():
 
 
 def test_apply_over_outputs():
+    """Test the `luigi_tools.util.apply_over_outputs` function."""
+
     class TaskA(luigi.Task):
+        """A simple test task."""
+
         a = luigi.Parameter(default="a")
 
         def output(self):
@@ -105,6 +122,8 @@ def test_apply_over_outputs():
 
 
 def test_dependency_graph(tmpdir, task_collection):
+    """Test dependency graph."""
+    # pylint: disable=use-implicit-booleaness-not-comparison
     start = task_collection.TaskE()
 
     # Test get_dependency_graph()
@@ -117,6 +136,12 @@ def test_dependency_graph(tmpdir, task_collection):
         (task_collection.TaskC(), task_collection.TaskA()),
     ]
 
+    # Test get_dependency_graph() with only one node
+    assert luigi_tools.util.get_dependency_graph(task_collection.TaskA()) == []
+    assert luigi_tools.util.get_dependency_graph(task_collection.TaskA(), allow_orphans=True) == [
+        (task_collection.TaskA(), None)
+    ]
+
     # Test graphviz_dependency_graph()
     dot = luigi_tools.util.graphviz_dependency_graph(graph)
     assert dot.body == [
@@ -143,8 +168,6 @@ def test_dependency_graph(tmpdir, task_collection):
     assert dot.edge_attr == {"arrowsize": "0.5", "style": "setlinewidth(0.5)"}
 
     # Test graphviz_dependency_graph() with custom attributes
-    from graphviz import Digraph
-
     dot_with_attrs = luigi_tools.util.graphviz_dependency_graph(
         graph,
         graph_attrs={"bgcolor": "red"},
@@ -186,6 +209,12 @@ def test_dependency_graph(tmpdir, task_collection):
     with pytest.raises(ValueError):
         luigi_tools.util.graphviz_dependency_graph([])
 
+    # Test graphviz_dependency_graph() with only one node
+    dot_one_node_orphan = luigi_tools.util.graphviz_dependency_graph(
+        [(task_collection.TaskA(), None)]
+    )
+    assert dot_one_node_orphan.source.count("node ") == 1
+
     # Test render_dependency_graph()
     output_file = Path(tmpdir / "test_dependency_graph.png")
     luigi_tools.util.render_dependency_graph(
@@ -208,6 +237,7 @@ def test_dependency_graph(tmpdir, task_collection):
 
 
 def test_param_repr():
+    """Test the `luigi_tools.util._param_repr` function."""
     assert luigi_tools.util._param_repr(None, PARAM_NO_VALUE) == ""
     assert luigi_tools.util._param_repr(None, None) == "(None)"
     assert luigi_tools.util._param_repr("description", None) == "description(None)"
@@ -215,8 +245,11 @@ def test_param_repr():
 
 
 class TestRegisterTemplates:
+    """Test the configuration template mechanism."""
+
     @pytest.fixture
     def config_reseter(self):
+        """Fixture to reset the luigi configuration."""
         cfg_cls = luigi.configuration.cfg_parser.LuigiConfigParser
         current_config = copy.deepcopy(cfg_cls._config_paths)
         yield
@@ -226,7 +259,11 @@ class TestRegisterTemplates:
 
     @pytest.fixture
     def Task(self, tmpdir):
+        """A simple test task."""
+
         class Task(luigi.Task):
+            """A simple test task."""
+
             a = luigi.Parameter(default="a")
             expected_a = luigi.Parameter(default="a")
 
@@ -240,15 +277,17 @@ class TestRegisterTemplates:
 
     @pytest.fixture
     def template_dir(self, tmpdir):
+        """Create a dummy template directory."""
         template_dir = tmpdir / "templates"
         template_dir.mkdir()
         config = ConfigParser()
         config.read_dict({"Task": {"a": "a_from_template"}})
-        with open(template_dir / "template_1.cfg", "w") as f:
+        with open(template_dir / "template_1.cfg", "w", encoding="utf-8") as f:
             config.write(f)
         return str(template_dir)
 
     def test_cfg_only(self, Task, template_dir, config_reseter):
+        """Test with only a config file."""
         with set_luigi_config(
             {
                 "Task": {"a": "a_from_cfg"},
@@ -258,6 +297,7 @@ class TestRegisterTemplates:
             assert luigi.build([Task(expected_a="a_from_cfg")], local_scheduler=True)
 
     def test_template_only(self, Task, template_dir, config_reseter):
+        """Test with only a template."""
         with set_luigi_config(
             {
                 "Template": {"name": "template_1"},
@@ -267,6 +307,7 @@ class TestRegisterTemplates:
             assert luigi.build([Task(expected_a="a_from_template")], local_scheduler=True)
 
     def test_template_and_cfg(self, Task, template_dir, config_reseter):
+        """Test with both a config file and a template."""
         with set_luigi_config(
             {
                 "Template": {"name": "template_1"},
@@ -277,6 +318,7 @@ class TestRegisterTemplates:
             assert luigi.build([Task(expected_a="a_from_cfg")], local_scheduler=True)
 
     def test_missing_template(self, Task, template_dir, config_reseter):
+        """Test with a missing template."""
         with set_luigi_config(
             {
                 "Template": {"name": "missing_template"},
@@ -287,6 +329,7 @@ class TestRegisterTemplates:
                 luigi_tools.util.register_templates(template_dir)
 
     def test_template_directory_in_cfg(self, Task, template_dir, config_reseter):
+        """Test with a template directory defined in the luigi config file."""
         with set_luigi_config(
             {
                 "Template": {
@@ -300,6 +343,7 @@ class TestRegisterTemplates:
             assert luigi.build([Task(expected_a="a_from_cfg")], local_scheduler=True)
 
     def test_template_directory_override(self, Task, template_dir, config_reseter):
+        """Test with a template directory defined in luigi.cfg but overridden in the task ctor."""
         directory = Path(template_dir)
         new_directory = directory.with_name("new_templates")
         directory.rename(new_directory)
@@ -315,6 +359,7 @@ class TestRegisterTemplates:
             assert luigi.build([Task(expected_a="a_from_template")], local_scheduler=True)
 
     def test_no_directory(self, Task, config_reseter):
+        """Test without template directory given to the template registering function."""
         with set_luigi_config(
             {
                 "Template": {"name": "template_name"},
@@ -328,6 +373,7 @@ class TestRegisterTemplates:
                 luigi_tools.util.register_templates()
 
     def test_no_name(self, Task, config_reseter):
+        """Test with no template name given."""
         with set_luigi_config(
             {
                 "Template": {"directory": "any directory"},
@@ -341,6 +387,7 @@ class TestRegisterTemplates:
                 luigi_tools.util.register_templates()
 
     def test_no_hierarchy_end(self, Task, template_dir, config_reseter):
+        """Test the hierarchy_end parameter."""
         # Register the template only
         luigi_tools.util.register_templates(template_dir, "template_1", hierarchy_end=False)
 
@@ -350,7 +397,7 @@ class TestRegisterTemplates:
         new_directory.mkdir()
         config = ConfigParser()
         config.read_dict({"Task": {"a": "a_from_2nd_template"}})
-        with open(new_directory / "template_2.cfg", "w") as f:
+        with open(new_directory / "template_2.cfg", "w", encoding="utf-8") as f:
             config.write(f)
 
         # Register the new template and the luigi.cfg file
@@ -359,9 +406,15 @@ class TestRegisterTemplates:
 
 
 class TestSetLuigiConfig:
+    """Test the `luigi_tools.util.set_luigi_config` function."""
+
     @pytest.fixture
     def Task(self, tmpdir):
+        """A simple test task."""
+
         class Task(luigi.Task):
+            """A simple test task."""
+
             a = luigi.Parameter(default="a")
             expected_a = luigi.Parameter(default="a")
 
@@ -374,12 +427,14 @@ class TestSetLuigiConfig:
         return Task
 
     def test_defaults(self, Task):
+        """Test with empty config."""
         assert luigi.build([Task()], local_scheduler=True)
 
         with set_luigi_config():
             assert luigi.build([Task()], local_scheduler=True)
 
     def test_new_config(self, Task):
+        """Test with new config."""
         with set_luigi_config(
             {
                 "Task": {"a": "a_from_cfg"},
@@ -408,6 +463,7 @@ class TestSetLuigiConfig:
         ]
 
     def test_config_file(self, Task):
+        """Test with a new config and a config file."""
         filename = "test_config_file.cfg"
         with set_luigi_config(
             {
@@ -420,6 +476,7 @@ class TestSetLuigiConfig:
         assert not Path(filename).exists()
 
     def test_params_ConfigParser(self, Task):
+        """Test using an `ConfigParser` instead of a `dict`.."""
         params = {
             "Task": {"a": "a_from_cfg"},
         }
@@ -430,6 +487,7 @@ class TestSetLuigiConfig:
 
 
 def test_deprecation_warning():
+    """Test the deprecation warnings."""
     with pytest.warns(
         luigi_tools.MovedToLuigiWarning,
         match="This feature was moved to the luigi package and is available since version 1.2.3.",
